#!/usr/bin/python
# -*- coding: utf-8 -*-
import html
import os
import re
from shutil import move
from typing import Generator

from saucenao.files.filter import Filter


class FileHandler:
    @staticmethod
    def get_files(directory, file_filter=None) -> Generator[str, None, None]:
        """Get all files from given directory

        :type directory: str
        :type file_filter: Filter
        :return:
        """
        if file_filter:
            for f in file_filter.apply(directory=directory):
                yield f
        else:
            for f in os.listdir(directory):
                if os.path.isfile(os.path.join(directory, f)):
                    yield f

    @staticmethod
    def unicode_translate(text, chars="", replacement="") -> str:
        """Replacement for the string.maketrans function

        :type text: str
        :type chars: str
        :type replacement: str
        :return:
        """
        for char in chars:
            text = text.replace(char, replacement[chars.index(char)])
        return text

    @staticmethod
    def move_to_category(filename, category, base_directory=os.getcwd()):
        """Move file to the sub_category folder

        :type base_directory: str
        :type filename: str
        :type category: str
        :return:
        """
        folder = re.sub(r'["/?*:<>|]', r'', html.unescape(category))
        folder = os.path.join(base_directory, folder)
        folder = os.path.abspath(FileHandler.unicode_translate(folder, "\n\t\r", "   "))
        if not os.path.exists(folder):
            os.makedirs(folder)
        move(os.path.join(base_directory, filename), os.path.join(folder, filename))



#!/usr/local/bin/python
# coding: utf-8
from tests.files.test_constraint import TestConstraint
from tests.files.test_filehandler import TestFileHandler
from tests.files.test_filter import TestFilesFilter

__all__ = [TestConstraint, TestFileHandler, TestFilesFilter]



#!/usr/bin/python
# -*- coding: utf-8 -*-
from typing import Callable


class Constraint:

    def __init__(self, value, cmp_func: Callable):
        """Initializing function

        :param value:
        :type cmp_func: Callable
        """
        self.value = value
        self.cmp_func = cmp_func

    @staticmethod
    def cmp_value_bigger(x, y):
        return x > y

    @staticmethod
    def cmp_value_bigger_or_equal(x, y):
        return x >= y

    @staticmethod
    def cmp_value_smaller(x, y):
        return x < y

    @staticmethod
    def cmp_value_smaller_or_equal(x, y):
        return x <= y

    @staticmethod
    def cmp_value_equals(x, y):
        return x == y

    @staticmethod
    def cmp_value_not_equals(x, y):
        return x != y



#!/usr/bin/python
# -*- coding: utf-8 -*-
import time

try:
    from titlesearch import get_similar_titles
except ImportError:
    get_similar_titles = None

from saucenao import SauceNao, FileHandler


class Worker(SauceNao):
    """
    Worker class for checking a list of files
    """

    def __init__(self, files, *args, **kwargs):
        """
        initializing function

        :type files: list|tuple|Generator
        :param args:
        :param kwargs:
        """
        super().__init__(*args, **kwargs)
        self.complete_file_list = files

    def run(self):
        """Check all files with SauceNao and execute the specified tasks

        :return:
        """
        for file_name in self.files:
            start_time = time.time()

            filtered_results = self.check_file(file_name)

            if not filtered_results:
                self.logger.info('No results found for image: {0:s}'.format(file_name))
                continue

            if self._move_to_categories:
                self.move_to_categories(file_name=file_name, results=filtered_results)
            else:
                yield {
                    'filename': file_name,
                    'results': filtered_results
                }

            duration = time.time() - start_time
            if duration < (30 / SauceNao.LIMIT_30_SECONDS):
                self.logger.debug("sleeping '{:.2f}' seconds".format((30 / SauceNao.LIMIT_30_SECONDS) - duration))
                time.sleep((30 / SauceNao.LIMIT_30_SECONDS) - duration)

    @property
    def excludes(self):
        """Property for excludes

        :return:
        """
        if self._exclude_categories:
            return [l.lower() for l in self._exclude_categories.split(",")]
        else:
            return []

    @property
    def files(self):
        """Property for files

        :return:
        """
        if self._start_file:
            # change files from generator to list
            files = list(self.complete_file_list)
            try:
                return files[files.index(self._start_file):]
            except ValueError:
                return self.complete_file_list
        return self.complete_file_list

    def get_category(self, results):
        """retrieve the category of the checked image based which can be either
        the content of the image or the author of the image

        :param results:
        :return: str
        """
        if self._use_author_as_category:
            categories = self.get_title_value(results, SauceNao.CONTENT_AUTHOR_KEY)
        else:
            categories = self.get_content_value(results, SauceNao.CONTENT_CATEGORY_KEY)

        if not categories:
            return ''

        self.logger.debug('categories: {0:s}'.format(', '.join(categories)))

        # since many pictures are tagged as original and with a proper category
        # we remove the original category if we have more than 1 category
        if not self._use_author_as_category and len(categories) > 1 and 'original' in categories:
            categories.remove('original')

        # take the first category
        return categories[0]

    def move_to_categories(self, file_name: str, results):
        """Check the file for categories and move it to the corresponding folder

        :type file_name: str
        :type results: list|tuple|Generator
        :return: bool
        """
        category = self.get_category(results)
        if not category:
            self.logger.info("no categories found for file: {0:s}".format(file_name))
            return False

        if not self._use_author_as_category:
            category = self.get_similar_title(category)

        # sub categories we don't want to move like original etc
        if category.lower() in self.excludes:
            self.logger.info("skipping excluded category: {0:s} ({1:s})".format(category, file_name))
            return False

        self.logger.info("moving {0:s} to category: {1:s}".format(file_name, category))
        FileHandler.move_to_category(file_name, category, base_directory=self._directory)
        return True

    def get_similar_title(self, category: str):
        """Check for a similar title of the category using my TitleSearch project which you can find here:
        https://github.com/DaRealFreak/TitleSearch

        :param category:
        :return:
        """
        if get_similar_titles:
            similar_titles = get_similar_titles(category)

            if similar_titles and similar_titles[0]['similarity'] * 100 >= self._title_minimum_similarity:
                self.logger.info(
                    "Similar title found: {0:s}, {1:s} ({2:.2f}%)".format(
                        category, similar_titles[0]['title'], similar_titles[0]['similarity'] * 100))
                return similar_titles[0]['title']

        return category



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import shutil
import sys
import types
import unittest
from uuid import uuid4

from PIL import Image

from saucenao import run_application


class TestInit(unittest.TestCase):
    SAUCENAO_MIN_WIDTH = 3
    SAUCENAO_MIN_HEIGHT = 3

    def setUp(self):
        """Constructor for unittest classes

        :return:
        """
        directory = str(uuid4())
        os.mkdir(directory)
        self.directory = os.path.abspath(directory)
        self.generate_small_jpg()

    def tearDown(self):
        """Destructor for unittest classes

        :return:
        """
        shutil.rmtree(self.directory)

    def generate_small_jpg(self):
        """Generate a rather small jpg file to upload faster(631 bytes)

        :return:
        """
        file_path = str(uuid4()) + ".jpg"
        im = Image.new("RGB", (self.SAUCENAO_MIN_WIDTH, self.SAUCENAO_MIN_HEIGHT))
        im.save(os.path.join(self.directory, file_path), "JPEG")
        return file_path

    def test_run_application(self):
        """Test the run_application function

        :return:
        """
        sys.argv = [sys.argv[0], '-d', self.directory]
        results = run_application()
        self.assertIsInstance(results, types.GeneratorType)
        results = list(results)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestInit)
    unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from saucenao.files.constraint import Constraint
from saucenao.files.filehandler import FileHandler
from saucenao.files.filter import Filter

__all__ = [Constraint, FileHandler, Filter]



#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
import logging

from saucenao.files import Constraint, FileHandler, Filter
from saucenao.saucenao import SauceNao
from saucenao.worker import Worker

__all__ = [SauceNao, FileHandler, Filter, Constraint]


def run_application():
    """Run SauceNao based on arguments passed to the file

    :return:
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--dir', help='directory to sort', required=True)
    parser.add_argument('-db', '--databases', default=999, type=int, help='which databases should be searched')
    parser.add_argument('-min', '--minimum-similarity', default=65, type=float,
                        help='minimum similarity percentage')
    parser.add_argument('-c', '--combine-api-types', action='store_true',
                        help='combine html and json api response to retrieve more information')
    parser.add_argument('-k', '--api-key', help='API key of your account on SauceNao')
    parser.add_argument('-x', '--exclude-categories', type=str, help='exclude specific categories from moving')
    parser.add_argument('-mv', '--move-to-categories', action='store_true', help='move images to categories')
    parser.add_argument('-author', '--use-author-as-category', default=False, action='store_true',
                        help='use author as category key instead of material')
    parser.add_argument('-o', '--output-type', default=0, type=int, help='0(html) or 2(json) API response')
    parser.add_argument('-sf', '--start-file',
                        help='with which file the checks start in case of after reaching the daily limit')
    parser.add_argument('-log', '--log-level', default=logging.ERROR, type=int,
                        help='which log level should be used, check logging._levelNames for options')

    parser.add_argument('-fcrdt', '--filter-creation-date', type=str,
                        help='filters files for created after given date. '
                             'Format of date has to match "d.m.Y[ H:M[:S]]"')
    parser.add_argument('-fmdt', '--filter-modified-date', type=str,
                        help='filters files for modified after given date. '
                             'Format of date has to match "d.m.Y[ H:M[:S]]"')

    parser.add_argument('-tmin', '--title-minimum-similarity', default=95, type=float,
                        help='minimum similarity percentage for title search with BakaUpdates, MyAnimeList and '
                             'VisualNovelDatabase')

    args = parser.parse_args()

    file_filter = Filter(assert_is_file=True)
    if args.filter_creation_date:
        file_filter._filter_creation_date = Constraint(value=args.filter_creation_date,
                                                       cmp_func=Constraint.cmp_value_bigger_or_equal)
    if args.filter_modified_date:
        file_filter._filter_modified_date = Constraint(value=args.filter_modified_date,
                                                       cmp_func=Constraint.cmp_value_bigger_or_equal)
    working_files = FileHandler.get_files(args.dir, file_filter)

    saucenao_worker = Worker(files=working_files, directory=args.dir, databases=args.databases,
                             minimum_similarity=args.minimum_similarity, combine_api_types=args.combine_api_types,
                             api_key=args.api_key, exclude_categories=args.exclude_categories,
                             move_to_categories=args.move_to_categories,
                             use_author_as_category=args.use_author_as_category, start_file=args.start_file,
                             log_level=args.log_level, title_minimum_similarity=args.title_minimum_similarity)
    return saucenao_worker.run()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import math
import os
import re
import shutil
import unittest
import uuid
from stat import ST_MTIME, ST_ATIME
from time import time, sleep

from saucenao.files.constraint import Constraint
from saucenao.files.filter import Filter


class TestFilesFilter(unittest.TestCase):
    """
    test cases for the filter in the files submodule
    """

    TEST_BIG_FILE_SIZE = 1024

    def setUp(self):
        """Constructor for the unittest

        :return:
        """
        self.dir = os.path.join(os.getcwd(), str(uuid.uuid4()))
        os.mkdir(self.dir)
        self.create_test_dir()
        # no files found for this time
        self.time_before_first_creation = time()
        sleep(0.1)
        self.create_big_file()
        # 1 file found
        self.time_after_first_creation = time()
        self.create_named_file()
        # we set the last modified time to 4 hours in the future, add a second
        # to get rid of other file operations
        self.time_modifying = time()
        self.create_modified_file()
        # wait for file operations to finish
        sleep(0.2)

    def tearDown(self):
        """Destructor for the unittest

        :return:
        """
        shutil.rmtree(self.dir)

    def test_files(self):
        """Test for files no folders

        :return:
        """
        expected_file_count = len([f for f in os.listdir(self.dir) if os.path.isfile(os.path.join(self.dir, f))])
        file_filter = Filter(assert_is_file=True)
        file_count = len(list(file_filter.apply(self.dir)))
        self.assertTrue(expected_file_count == file_count)

    def test_folders(self):
        """Test for folders no files

        :return:
        """
        expected_folder_count = len([f for f in os.listdir(self.dir) if os.path.isdir(os.path.join(self.dir, f))])
        folder_filter = Filter(assert_is_folder=True)
        folder_count = len(list(folder_filter.apply(self.dir)))
        self.assertTrue(folder_count == expected_folder_count)

    def test_named_file(self):
        """Test for regex match

        :return:
        """

        def test_cmp(value, expected_value):
            return expected_value.match(value)

        # check if we have more than just 1 file in the directory
        self.assertTrue(len(os.listdir(self.dir)) > 1)
        # set regular expression constraint
        file_filter = Filter(name=Constraint(re.compile('named*'), cmp_func=test_cmp))
        files = file_filter.apply(directory=self.dir)
        self.assertEqual(len(list(files)), 1)

    def test_extension_file(self):
        """Test filter for file extension

        :return:
        """

        def test_cmp(value, expected_value):
            return value in expected_value

        # we named our named file: named_file.jpg
        file_filter = Filter(file_type=Constraint([".png"], cmp_func=test_cmp))
        files = file_filter.apply(directory=self.dir)
        self.assertEqual(len(list(files)), 0)

        file_filter = Filter(file_type=Constraint([".jpg", ".png"], cmp_func=test_cmp))
        files = file_filter.apply(directory=self.dir)
        self.assertEqual(len(list(files)), 1)

    def test_creation_date(self):
        """Test for filtering after creation date

        :return:
        """
        date_string = datetime.datetime.fromtimestamp(self.time_modifying).strftime('%d.%m.%Y %H:%M')
        file_filter = Filter(
            creation_date=Constraint(date_string, cmp_func=Constraint.cmp_value_bigger_or_equal))
        files = file_filter.apply(directory=self.dir)
        self.assertEqual(len(list(files)), 4)
        file_filter = Filter(
            creation_date=Constraint(date_string, cmp_func=Constraint.cmp_value_smaller))
        files = file_filter.apply(directory=self.dir)
        self.assertEqual(len(list(files)), 0)

    def test_modified_file(self):
        """Test for modified file

        :return:
        """
        date_string = datetime.datetime.fromtimestamp(self.time_modifying).strftime('%d.%m.%Y %H:%M:%S')

        # get files created before or at the same time
        file_filter = Filter(
            modified_date=Constraint(date_string, cmp_func=Constraint.cmp_value_smaller_or_equal))
        files = file_filter.apply(directory=self.dir)
        len_files_before_modified_date = len(list(files))

        # get files modified after last file got created
        file_filter = Filter(
            modified_date=Constraint(date_string, cmp_func=Constraint.cmp_value_bigger))
        files = file_filter.apply(directory=self.dir)
        len_files_after_modified_date = len(list(files))

        self.assertTrue(len_files_after_modified_date == 1)
        self.assertTrue(len_files_before_modified_date > len_files_after_modified_date)

    def test_big_file(self):
        """Test for file size filter

        :return:
        """
        # check if we have more than just 1 file in the directory
        self.assertTrue(len(os.listdir(self.dir)) > 1)
        # filter now for files >= 1024 bytes
        file_filter = Filter(size=Constraint(self.TEST_BIG_FILE_SIZE, cmp_func=Constraint.cmp_value_bigger_or_equal))
        files = file_filter.apply(directory=self.dir)
        file_list = list(files)
        # workaround since scrutinizer has another file in created directories with differencing sizes each run
        # normal check would've been to check the file list length == 1
        self.assertTrue('big_file' in file_list and 'named_file.jpg' not in file_list)

    def test_non_existent_path(self):
        """Test filter for removing non existent files

        :return:
        """
        file_filter = Filter()
        files = list(file_filter.apply(directory=self.dir, file_system_objects=['not-existent-file']))
        self.assertEqual(files, [])

    def test_empty_filter(self):
        """Test empty filter

        :return:
        """
        file_filter = Filter()
        files = list(file_filter.apply())
        self.assertEqual(files, [])

    def test_get_timestamp_from_datestring(self):
        """Test for date string to timestamp conversion

        :return:
        """
        # since I don't like timezones in programming, I'll just expect a returned floating type value
        self.assertIsInstance(Filter._get_timestamp_from_datestring("01.01.2017 12:45:45"), float)
        self.assertIsInstance(Filter._get_timestamp_from_datestring("01.01.2017 12:45"), float)
        self.assertIsInstance(Filter._get_timestamp_from_datestring("01.01.2017"), float)
        with self.assertRaises(AttributeError) as _:
            Filter._get_timestamp_from_datestring("this is no time string")

    def create_big_file(self):
        """Create a file with a fixed size

        :return:
        """
        file_name = os.path.join(self.dir, 'big_file')
        with open(file_name, "wb") as file_handler:
            file_handler.seek(self.TEST_BIG_FILE_SIZE - 1)
            file_handler.write(b"\0")

    def create_named_file(self):
        """Create a file with a specific name

        :return:
        """
        file_name = os.path.join(self.dir, 'named_file.jpg')
        with open(file_name, "wb") as _:
            pass

    def create_modified_file(self):
        """Create a file and change the modification date

        :return:
        """
        file_name = os.path.join(self.dir, str(uuid.uuid4()))
        # create the file
        with open(file_name, "wb") as file_handler:
            file_handler.write(b"\0")

        st = os.stat(file_name)
        access_time = st[ST_ATIME]
        modified_time = st[ST_MTIME]

        os.utime(file_name, (access_time, modified_time + (4 * 3600)))

    def create_test_dir(self):
        """Create a subfolder for testing the folder amount

        :return:
        """
        dir_path = os.path.join(self.dir, str(uuid.uuid4()))
        os.mkdir(dir_path)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestFilesFilter)
    unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest
import uuid
from time import sleep

from saucenao import FileHandler, Filter


class TestFileHandler(unittest.TestCase):
    """
    test cases for the file handler operations
    """

    def setUp(self):
        """Constructor for the unittest

        :return:
        """
        self.dir = os.path.join(os.getcwd(), str(uuid.uuid4()))
        os.mkdir(self.dir)

        file_name = os.path.join(self.dir, str(uuid.uuid4()))
        # create a file
        with open(file_name, "wb") as _:
            pass

        # wait for file operations to finish
        sleep(0.5)

    def tearDown(self):
        """Destructor for the unittest

        :return:
        """
        shutil.rmtree(self.dir)

    def test_get_files(self):
        """Test the get_files function of the file handler

        :return:
        """
        files = FileHandler.get_files(directory=self.dir)
        self.assertEqual(len(list(files)), 1)

        file_filter = Filter(assert_is_folder=True)
        files = FileHandler.get_files(directory=self.dir, file_filter=file_filter)
        self.assertEqual(len(list(files)), 0)

        file_filter = Filter(assert_is_file=True)
        files = FileHandler.get_files(directory=self.dir, file_filter=file_filter)
        self.assertEqual(len(list(files)), 1)

    def test_move_to_category(self):
        """Test moving the file to a category folder

        :return:
        """
        files = list(FileHandler.get_files(directory=self.dir))
        self.assertEqual(len(files), 1)
        for file in files:
            FileHandler.move_to_category(file, 'Example Category', self.dir)

        new_files = FileHandler.get_files(directory=self.dir)
        self.assertEqual(len(list(new_files)), 0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import logging
import os
import re
import time
from mimetypes import MimeTypes
from typing import Generator

import requests
from bs4 import BeautifulSoup as Soup
from bs4 import element

from saucenao import http
from saucenao.exceptions import *


class SauceNao(object):
    """"
    small script to work with SauceNao locally
    """

    SEARCH_POST_URL = 'http://saucenao.com/search.php'

    # basic account allows currently 20 images within 30 seconds
    # you can increase this value is you have a premium account
    LIMIT_30_SECONDS = 20

    # 0=html, 2=json but json is omitting important data but includes more data about authors
    # taken from the API documentation(requires login): https://saucenao.com/user.php?page=search-api
    API_HTML_TYPE = 0
    API_JSON_TYPE = 2

    CONTENT_CATEGORY_KEY = 'Material'
    CONTENT_AUTHOR_KEY = 'Creator'
    CONTENT_CHARACTERS_KEY = 'Characters'

    mime = None
    logger = None

    def __init__(self, directory, databases=999, minimum_similarity=65, combine_api_types=False, api_key=None,
                 exclude_categories='', move_to_categories=False, use_author_as_category=False,
                 output_type=API_HTML_TYPE, start_file=None, log_level=logging.ERROR, title_minimum_similarity=90):
        """Initializing function

        :type directory: str
        :type databases: int
        :type minimum_similarity: float
        :type combine_api_types: bool
        :type api_key: str
        :type exclude_categories: str
        :type move_to_categories: bool
        :type use_author_as_category: bool
        :type output_type: int
        :type start_file: str
        :type log_level: int
        :type title_minimum_similarity: float
        """
        self._directory = directory
        self._databases = databases
        self._minimum_similarity = minimum_similarity
        self._combine_api_types = combine_api_types
        self._api_key = api_key
        self._exclude_categories = exclude_categories
        self._move_to_categories = move_to_categories
        self._use_author_as_category = use_author_as_category
        self._output_type = output_type
        self._start_file = start_file
        self._title_minimum_similarity = title_minimum_similarity

        self._previous_status_code = None

        self.mime = MimeTypes()
        logging.basicConfig(level=log_level)
        self.logger = logging.getLogger("saucenao_logger")

    def check_file(self, file_name: str) -> list:
        """Check the given file for results on SauceNAO

        :type file_name: str
        :return:
        """
        self.logger.info("checking file: {0:s}".format(file_name))
        if self._combine_api_types:
            result = self.check_image(file_name, self.API_HTML_TYPE)
            sorted_results = self.parse_results_json(result)

            additional_result = self.check_image(file_name, self.API_JSON_TYPE)
            additional_sorted_results = self.parse_results_json(additional_result)
            sorted_results = self.merge_results(sorted_results, additional_sorted_results)
        else:
            result = self.check_image(file_name, self._output_type)
            sorted_results = self.parse_results_json(result)

        filtered_results = self.filter_results(sorted_results)
        return filtered_results

    def get_http_data(self, file_path: str, output_type: int):
        """Prepare the http relevant data(files, headers, params) for the given file path and output type

        :param file_path:
        :param output_type:
        :return:
        """
        with open(file_path, 'rb') as file_object:
            files = {'file': file_object.read()}

        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                          'Chrome/63.0.3239.84 Safari/537.36',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
            'Accept-Language': 'en-DE,en-US;q=0.9,en;q=0.8',
            'Accept-Encoding': 'gzip, deflate, br',
            'DNT': '1',
            'Connection': 'keep-alive'
        }

        params = {
            'file': file_path,
            'Content-Type': self.mime.guess_type(file_path),
            # parameters taken from form on main page: https://saucenao.com/
            'url': None,
            'frame': 1,
            'hide': 0,
            # parameters taken from API documentation: https://saucenao.com/user.php?page=search-api
            'output_type': output_type,
            'db': self._databases,
        }

        if self._api_key:
            params['api_key'] = self._api_key

        return files, params, headers

    def check_image(self, file_name: str, output_type: int) -> str:
        """Check the possible sources for the given file

        :type output_type: int
        :type file_name: str
        :return:
        """
        file_path = os.path.join(self._directory, file_name)

        files, params, headers = self.get_http_data(file_path=file_path, output_type=output_type)
        link = requests.post(url=self.SEARCH_POST_URL, files=files, params=params, headers=headers)

        code, msg = http.verify_status_code(link, file_name)

        if code == http.STATUS_CODE_SKIP:
            self.logger.error(msg)
            return json.dumps({'results': []})
        elif code == http.STATUS_CODE_REPEAT:
            if not self._previous_status_code:
                self._previous_status_code = code
                self.logger.info("Received an unexpected status code, repeating after 10 seconds...")
                time.sleep(10)
                return self.check_image(file_name, output_type)
            else:
                raise UnknownStatusCodeException(msg)
        else:
            self._previous_status_code = None

        if output_type == self.API_HTML_TYPE:
            return self.parse_results_html_to_json(link.text)

        return link.text

    @staticmethod
    def parse_results_html_to_json(html: str) -> str:
        """Parse the results and sort them descending by similarity

        :type html: str
        :return:
        """
        soup = Soup(html, 'html.parser')
        # basic format of json API response
        results = {'header': {}, 'results': []}

        for res in soup.find_all('td', attrs={"class": "resulttablecontent"}):  # type: element.Tag
            # optional field in SauceNao
            title_tag = res.find_next('div', attrs={"class": "resulttitle"})
            if title_tag:
                title = title_tag.text
            else:
                title = ''

            # mandatory field in SauceNao
            similarity = res.find_next('div', attrs={"class": "resultsimilarityinfo"}).text.replace('%', '')
            alternate_links = [a_tag['href'] for a_tag in
                               res.find_next('div', attrs={"class": "resultmiscinfo"}).find_all('a', href=True)]
            content_column = []
            content_column_tags = res.find_all('div', attrs={"class": "resultcontentcolumn"})
            for content_column_tag in content_column_tags:
                for br in content_column_tag.find_all('br'):
                    br.replace_with('\n')
                content_column.append(content_column_tag.text)

            result = {
                'header': {
                    'similarity': similarity
                },
                'data': {
                    'title': title,
                    'content': content_column,
                    'ext_urls': alternate_links
                }
            }
            results['results'].append(result)

        return json.dumps(results)

    @staticmethod
    def parse_results_json(text: str) -> list:
        """Parse the results and sort them descending by similarity

        :type text: str
        :return:
        """
        result = json.loads(text)
        results = [res for res in result['results']]
        return sorted(results, key=lambda k: float(k['header']['similarity']), reverse=True)

    def filter_results(self, sorted_results) -> list:
        """Return results with a similarity bigger or the same as the defined similarity from the arguments (default 65%)

        :type sorted_results: list|tuple|Generator
        :return:
        """
        filtered_results = []
        for res in sorted_results:
            if float(res['header']['similarity']) >= float(self._minimum_similarity):
                filtered_results.append(res)
            else:
                # we can break here since the results are sorted by similarity anyways
                break
        return filtered_results

    @staticmethod
    def get_content_value(results, key: str):
        """Return the first match of Material in content
        multiple sites have a categorisation which SauceNao utilizes to provide it in the content section

        :type results: list|tuple|Generator
        :type key: str
        :return:
        """
        for result in results:
            if 'content' in list(result['data'].keys()):
                for content in result['data']['content']:
                    if re.match('{0:s}: .*'.format(key), content):
                        return ''.join(re.split(r'{0:s}: '.format(key), content)[1:]).rstrip("\n").split('\n')
        return ''

    @staticmethod
    def get_title_value(results, key: str):
        """Return the first match of Material in the title section
        SauceNAO provides the authors name in the title section f.e. if provided by the indexed entry

        :type results: list|tuple|Generator
        :type key: str
        :return:
        """
        for result in results:
            if 'title' in list(result['data'].keys()):
                if re.match('{0:s}: .*'.format(key), result['data']['title']):
                    return ''.join(re.split(r'{0:s}: '.format(key), result['data']['title'])[1:]).rstrip("\n") \
                        .split('\n')
        return ''

    @staticmethod
    def merge_two_dicts(x: dict, y: dict) -> dict:
        """Take x dictionary and insert/overwrite y dictionary values

        :type x: dict
        :type y: dict
        :return:
        """
        z = x.copy()
        z.update(y)
        return z

    def merge_results(self, result: list, additional_result: list) -> list:
        """Merge two result arrays

        :type result: list
        :type additional_result: list
        :return:
        """
        if len(result) <= len(additional_result):
            length = len(result)
        else:
            length = len(additional_result)

        for i in range(length):
            for key in list(result[i].keys()):
                result[i][key] = self.merge_two_dicts(result[i][key], additional_result[i][key])

        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
import unittest

from saucenao.files.constraint import Constraint


class TestConstraint(unittest.TestCase):
    """
    test cases for the constraints in the files submodule
    """

    def test_constraint(self):
        """Test the initializing function of Constraint

        :return:
        """
        constraint = Constraint(value='Value', cmp_func=self)
        self.assertEqual(constraint.value, 'Value')
        self.assertEqual(constraint.cmp_func, self)

    def test_value_bigger(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_bigger(1, 2), False)
        self.assertEqual(Constraint.cmp_value_bigger(2, 2), False)
        self.assertEqual(Constraint.cmp_value_bigger(2, 1), True)

    def test_value_bigger_or_equal(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_bigger_or_equal(1, 2), False)
        self.assertEqual(Constraint.cmp_value_bigger_or_equal(2, 2), True)
        self.assertEqual(Constraint.cmp_value_bigger_or_equal(2, 1), True)

    def test_value_smaller(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_smaller(1, 2), True)
        self.assertEqual(Constraint.cmp_value_smaller(2, 2), False)
        self.assertEqual(Constraint.cmp_value_smaller(2, 1), False)

    def test_value_smaller_or_equal(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_smaller_or_equal(1, 2), True)
        self.assertEqual(Constraint.cmp_value_smaller_or_equal(2, 2), True)
        self.assertEqual(Constraint.cmp_value_smaller_or_equal(2, 1), False)

    def test_value_equals(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_equals(1, 2), False)
        self.assertEqual(Constraint.cmp_value_equals(2, 2), True)
        self.assertEqual(Constraint.cmp_value_equals(2, 1), False)

    def test_value_not_equals(self):
        """Test compare function cmp_value_bigger

        :return:
        """
        self.assertEqual(Constraint.cmp_value_not_equals(1, 2), True)
        self.assertEqual(Constraint.cmp_value_not_equals(2, 2), False)
        self.assertEqual(Constraint.cmp_value_not_equals(2, 1), True)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestConstraint)
    unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import logging
import os
import unittest
import uuid

import dotenv
from PIL import Image

from saucenao.saucenao import DailyLimitReachedException
from saucenao.saucenao import SauceNao

dotenv_path = os.path.join(os.path.dirname(__file__), os.path.pardir, os.path.pardir, '.env')
dotenv.load_dotenv(dotenv_path)

SAUCENAO_MIN_WIDTH = 3
SAUCENAO_MIN_HEIGHT = 3

SAUCENAO_IP_LIMIT_UNREGISTERED_USER = 150
SAUCENAO_IP_LIMIT_BASIC_USER = 300

SAUCENAO_API_KEY = os.environ.get('SAUCENAO_API_KEY')


def generate_small_jpg():
    """Generate a rather small jpg file to upload faster(631 bytes)

    :return:
    """
    file_path = str(uuid.uuid4()) + ".jpg"
    im = Image.new("RGB", (SAUCENAO_MIN_WIDTH, SAUCENAO_MIN_HEIGHT))
    im.save(os.path.join(os.getcwd(), file_path), "JPEG")
    return file_path


class TestSauceNaoLimits(unittest.TestCase):
    """
    test cases to check the limits for SauceNAO

    currently not covered is the test case: account limit reached, ip limit not reached
    have to add a proxy for this test case to be covered.

    Test cases covered:
     - HTML response type: ip/account limit not reached
     - JSON response type: ip/account limit not reached
     - HTML response type: ip limit reached
     - JSON response type: ip limit reached
    """

    def setUp(self):
        """Constructor for the unittest

        :return:
        """
        self.test_jpg = generate_small_jpg()

        self.saucenao_html = SauceNao(os.getcwd(), output_type=SauceNao.API_HTML_TYPE, log_level=logging.DEBUG)
        self.saucenao_json = SauceNao(os.getcwd(), output_type=SauceNao.API_JSON_TYPE, log_level=logging.DEBUG)

        self.NOT_IP_LIMIT_NOT_ACCOUNT_LIMIT = [
            {'function': self.check_response_no_api_key, 'expected_success': True},
            {'function': self.check_response_api_key, 'expected_success': True}
        ]
        self.IP_LIMIT_NOT_ACCOUNT_LIMIT = [
            {'function': self.check_response_no_api_key, 'expected_success': False},
            {'function': self.check_response_api_key, 'expected_success': True}
        ]
        self.IP_LIMIT_ACCOUNT_LIMIT = [
            {'function': self.check_response_no_api_key, 'expected_success': False},
            {'function': self.check_response_api_key, 'expected_success': False}
        ]

    def tearDown(self):
        """Destructor for the unittest

        :return:
        """
        os.remove(self.test_jpg)

    def run_tests(self, saucenao: SauceNao, tests):
        """Run the different tests with the given SauceNao instance

        :type saucenao: SauceNao
        :type tests: list|tuple|Generator
        :return:
        """
        for test in tests:
            test_function = test['function']
            test_result = test['expected_success']
            try:
                test_function(saucenao, assert_success=test_result)
            except Exception as e:
                self.fail("{} failed ({}: {})".format(test, type(e), e))

    def test_limits(self):
        """Test the limits of SauceNAO

        :return:
        """
        self.saucenao_html.logger.info('running HTML test, ip limit not reached, account limit not reached')
        self.run_tests(saucenao=self.saucenao_html, tests=self.NOT_IP_LIMIT_NOT_ACCOUNT_LIMIT)
        self.saucenao_html.logger.info('running JSON test, ip limit not reached, account limit not reached')
        self.run_tests(saucenao=self.saucenao_json, tests=self.NOT_IP_LIMIT_NOT_ACCOUNT_LIMIT)

        # now reach the daily limit without API key to reach the IP limit
        if self.saucenao_html._api_key:
            self.saucenao_html._api_key = None

        test_files = [self.test_jpg] * (SAUCENAO_IP_LIMIT_UNREGISTERED_USER - 2)
        try:
            # check_files returns a generator so we have to improvise here a bit
            for test_file in test_files:
                self.saucenao_html.check_file(test_file)
        except DailyLimitReachedException:
            pass

        # we are at 150 searches -> IP limit unregistered user reached, not basic user
        self.saucenao_html.logger.info('running HTML test, ip limit reached, account limit not reached')
        self.run_tests(saucenao=self.saucenao_html, tests=self.IP_LIMIT_NOT_ACCOUNT_LIMIT)
        self.saucenao_html.logger.info('running JSON test, ip limit reached, account limit not reached')
        self.run_tests(saucenao=self.saucenao_json, tests=self.IP_LIMIT_NOT_ACCOUNT_LIMIT)

        # set API key to reach the account limit
        self.saucenao_html._api_key = SAUCENAO_API_KEY
        test_files = [self.test_jpg] * (SAUCENAO_IP_LIMIT_BASIC_USER - SAUCENAO_IP_LIMIT_UNREGISTERED_USER - 4)
        try:
            # check_files returns a generator so we have to improvise here a bit
            for test_file in test_files:
                self.saucenao_html.check_file(test_file)
        except DailyLimitReachedException:
            pass

        # we are at 30 searches -> IP limit basic user reached
        self.saucenao_html.logger.info('running HTML test, ip limit reached, account limit reached')
        self.run_tests(saucenao=self.saucenao_html, tests=self.IP_LIMIT_ACCOUNT_LIMIT)
        self.saucenao_html.logger.info('running JSON test, ip limit reached, account limit reached')
        self.run_tests(saucenao=self.saucenao_json, tests=self.IP_LIMIT_ACCOUNT_LIMIT)

    def check_response_no_api_key(self, saucenao: SauceNao, assert_success=True):
        """Check the response without an API key
        on assert_success=True expect a dictionary, else an exception

        :type saucenao: SauceNao
        :type assert_success: bool
        :return:
        """
        if assert_success:
            result = saucenao.check_file(self.test_jpg)
            self.assertIsInstance(result, list)
        else:
            self.assertRaises(DailyLimitReachedException, saucenao.check_file, self.test_jpg)

    def check_response_api_key(self, saucenao: SauceNao, assert_success=True):
        """Check the response with an API key
        on assert_success=True expect a dictionary, else an exception

        :type saucenao: SauceNao
        :type assert_success: bool
        :return:
        """
        saucenao._api_key = SAUCENAO_API_KEY
        if assert_success:
            result = saucenao.check_file(self.test_jpg)
            self.assertIsInstance(result, list)
        else:
            self.assertRaises(DailyLimitReachedException, saucenao.check_file, self.test_jpg)


suite = unittest.TestLoader().loadTestsFromTestCase(TestSauceNaoLimits)
unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import os
import re
from pathlib import Path
from stat import ST_CTIME, ST_MTIME, ST_SIZE
from typing import Generator

from saucenao.files.constraint import Constraint


class Filter:
    """
    Filter to apply constraints for multiple metadata entries of the files.
    Obviously the files have to exist or they'll automatically get removed.
    Returns a generator object
    """

    _directory = None
    _file_system_objects = None

    def __init__(self, assert_is_folder=False, assert_is_file=False, creation_date=None, modified_date=None, name=None,
                 file_type=None, size=None):
        """Initializing function

        :type assert_is_folder: bool
        :type assert_is_file: bool
        :type creation_date: Constraint
        :type modified_date: Constraint
        :type name: Constraint
        :type file_type: Constraint
        :type size: Constraint
        """
        self._filter_assert_is_folder = assert_is_folder
        self._filter_assert_is_file = assert_is_file
        self._filter_creation_date = creation_date
        self._filter_modified_date = modified_date
        self._filter_name = name
        self._filter_file_type = file_type
        self._filter_file_size = size

    @property
    def file_system_objects(self):
        """Property for file system objects

        :return:
        """
        if self._file_system_objects is None:
            if self._directory:
                return os.listdir(self._directory)
            else:
                return []
        return self._file_system_objects

    @staticmethod
    def _get_timestamp_from_datestring(date_string) -> float:
        """Convert the given date string to timestamp

        :param date_string:
        :return:
        """
        if re.match(r'\d+.\d+.\d+ \d+:\d+:\d+', date_string):
            return datetime.datetime.strptime(date_string, "%d.%m.%Y %H:%M:%S").timestamp()
        elif re.match(r'\d+.\d+.\d+ \d+:\d+', date_string):
            return datetime.datetime.strptime(date_string, "%d.%m.%Y %H:%M").timestamp()
        elif re.match(r'\d+.\d+.\d+', date_string):
            return datetime.datetime.strptime(date_string, "%d.%m.%Y").timestamp()
        else:
            raise AttributeError("The date doesn't fit the format: d.m.Y[ H:M[:S]]")

    def apply(self, directory='', file_system_objects=None) -> Generator[str, None, None]:
        """Apply the filter values to the given FSOs(File System Objects)

        :type directory: str
        :type file_system_objects: list|tuple|Generator
        :return:
        """
        self._directory = directory
        self._file_system_objects = file_system_objects

        for file_system_object in self.file_system_objects:
            abs_path = os.path.join(directory, file_system_object)

            # check if the FSO exists, else we can't access the metadata
            if not os.path.exists(abs_path):
                continue

            # check if the FSO is a file
            if self._filter_assert_is_file and not os.path.isfile(abs_path):
                continue

            # check if the FSO is a folder
            if self._filter_assert_is_folder and not os.path.isdir(abs_path):
                continue

            file_stats = os.stat(abs_path)

            # check if the FSO creation date matches the constraint
            if self._filter_creation_date and not self.apply_creation_date(file_stats):
                continue

            # check if the FSO modification date matches the constraint
            if self._filter_modified_date and not self.apply_modified_date(file_stats):
                continue

            # check if the FSO name matches the constraint
            if self._filter_name and not self._filter_name.cmp_func(file_system_object, self._filter_name.value):
                continue

            # check if the FSO suffix matches the constraint
            if self._filter_file_type and not self._filter_file_type.cmp_func(Path(abs_path).suffix,
                                                                              self._filter_file_type.value):
                continue

            # check if the FSO size matches the constraint
            if self._filter_file_size and not self._filter_file_size.cmp_func(file_stats[ST_SIZE],
                                                                              self._filter_file_size.value):
                continue

            yield file_system_object

    def apply_creation_date(self, file_stats: os.stat_result) -> bool:
        """Apply creation date option

        :param file_stats:
        :return:
        """
        creation_time = file_stats[ST_CTIME]
        expected_creation_time = self._get_timestamp_from_datestring(self._filter_creation_date.value)
        if self._filter_creation_date.cmp_func(creation_time, expected_creation_time):
            return True
        else:
            return False

    def apply_modified_date(self, file_stats: os.stat_result) -> bool:
        """Apply modified date option

        :param file_stats:
        :return:
        """
        modified_time = file_stats[ST_MTIME]
        expected_modified_time = self._get_timestamp_from_datestring(self._filter_modified_date.value)
        if self._filter_modified_date.cmp_func(modified_time, expected_modified_time):
            return True
        else:
            return False



#!/usr/bin/python
# -*- coding: utf-8 -*-


class DailyLimitReachedException(Exception):
    pass


class InvalidOrWrongApiKeyException(Exception):
    pass


class UnknownStatusCodeException(Exception):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages

current_directory = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(current_directory, 'saucenao', '__version__.py'), 'r') as f:
    exec(f.read(), about)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name=about['__title__'],
      version=about['__version__'],
      description=about['__description__'],
      long_description=long_description,
      long_description_content_type="text/markdown",
      url=about['__url__'],
      author=about['__author__'],
      author_email=about['__author_email__'],
      license=about['__license__'],
      packages=find_packages(),
      install_requires=[
          'bs4>=0.0.1',
          'requests>=2.18.4'
      ],
      extras_require={
          'titlesearch': [
              'titlesearch>=0.0.1'
          ]
      },
      zip_safe=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import unittest

import requests_mock

from saucenao.http import *


class TestHttp(unittest.TestCase):
    """Test the verify_status_code function with multiple mock responses"""

    def setUp(self):
        """set up for unittests

        :return:
        """
        self.dummy_url = "mock://localhost"

    @requests_mock.mock()
    def test_status_code_ok(self, mock):
        """Test the verify status code function for 200 ok status code

        :return:
        """
        mock.get(self.dummy_url, status_code=200)
        status_code, msg = verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
        self.assertEqual(status_code, STATUS_CODE_OK)

    @requests_mock.mock()
    def test_status_code_skip(self, mock):
        """Test the verify status code function for 413 payload too large status code

        :return:
        """
        mock.get(self.dummy_url, status_code=413)
        status_code, msg = verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
        self.assertEqual(status_code, STATUS_CODE_SKIP)

    @requests_mock.mock()
    def test_status_code_repeat(self, mock):
        """Test the verify status code function with multiple mock request responses

        :return:
        """
        mock.get(self.dummy_url, text='', status_code=999)
        status_code, msg = verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
        self.assertEqual(status_code, STATUS_CODE_REPEAT)

    @requests_mock.mock()
    def test_status_code_api_key(self, mock):
        """Test the verify status code function for 403 wrong api key status

        :return:
        """
        mock.get(self.dummy_url, text='', status_code=403)
        with self.assertRaises(InvalidOrWrongApiKeyException) as _:
            verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')

    @requests_mock.mock()
    def test_status_code_limit(self, mock):
        """Test the verify status code function for 429 limit reached status

        :return:
        """
        mock.get(self.dummy_url, text='limit of 150 searches reached', status_code=429)
        with self.assertRaises(DailyLimitReachedException) as exception:
            verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
            self.assertEqual(exception.msg, 'Daily search limit for unregistered users reached')

        mock.get(self.dummy_url, text='limit of 300 searches reached', status_code=429)
        with self.assertRaises(DailyLimitReachedException) as exception:
            verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
            self.assertEqual(exception.msg, 'Daily search limit for basic users reached')

        mock.get(self.dummy_url, status_code=429)
        with self.assertRaises(DailyLimitReachedException) as exception:
            verify_status_code(request_response=requests.get(self.dummy_url), file_name='tmp')
            self.assertEqual(exception.msg, 'Daily search limit reached')


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestHttp)
    unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-

__title__ = 'SauceNAO'
__description__ = 'Small module to work with SauceNAO locally'
__version__ = '1.0.1'
__url__ = 'https://github.com/DaRealFreak/saucenao'
__author__ = 'DaRealFreak'
__author_email__ = 'steffen.keuper@web.de'
__license__ = 'MIT'



#!/usr/bin/python
# -*- coding: utf-8 -*-
import re
import unittest

from saucenao import __version__ as saucenao_version


class TestVersion(unittest.TestCase):
    """Test the version information for Python PEP standard"""

    def test_version_format(self):
        """Test the version number for a PyPi valid format according to PEP 440
        https://www.python.org/dev/peps/pep-0440/

        :return:
        """
        # ToDo: check all requirements
        self.assertEqual(bool(re.match('\d+.\d+', saucenao_version.__version__)), True)


if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(TestVersion)
    unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import requests

from saucenao.exceptions import *

PREVIOUS_STATUS_CODE = None

STATUS_CODE_OK = 1
STATUS_CODE_SKIP = 2
STATUS_CODE_REPEAT = 3


def verify_status_code(request_response: requests.Response, file_name: str) -> tuple:
    """Verify the status code of the post request to the search url and raise exceptions if the code is unexpected

    :type request_response: requests.Response
    :type file_name: str
    :return:
    """
    if request_response.status_code == 200:
        return STATUS_CODE_OK, ''

    elif request_response.status_code == 429:
        if 'limit of 150 searches' in request_response.text:
            raise DailyLimitReachedException('Daily search limit for unregistered users reached')
        elif 'limit of 300 searches' in request_response.text:
            raise DailyLimitReachedException('Daily search limit for basic users reached')
        else:
            raise DailyLimitReachedException('Daily search limit reached')
    elif request_response.status_code == 403:
        raise InvalidOrWrongApiKeyException("Invalid or wrong API key")
    elif request_response.status_code == 413:
        msg = "Payload too large, skipping file: {0:s}".format(file_name)
        return STATUS_CODE_SKIP, msg
    else:
        msg = "Unknown status code: {0:d}".format(request_response.status_code)
        return STATUS_CODE_REPEAT, msg
